

--[[ Addon object ]]--
Nefpad = {
	name    = "Nefpad",
	author  = "Nefarion",
	version = GetAddOnMetadata("Nefpad","Version") or "0",
	path    = "Interface\\AddOns\\Nefpad\\",
}
local Nefpad = Nefpad



--[[ Locale ]]--
local L = setmetatable(Nefpad_Locale or {},{  -- grab the locale table from the global.
	__index = function(tbl,key)
		tbl[key] = "NOT LOCALIZED"  -- just so it doesn't error out on unsuppored locales.
		return "NOT LOCALIZED"
	end,
})
Nefpad.locale = L  -- save it globally in our namespace.
Nefpad_Locale = nil  -- clear our unused global.



--[[ Saved vars ]]--
do
	local eFrame = CreateFrame("Frame")
	eFrame:RegisterEvent("VARIABLES_LOADED")
	eFrame:SetScript("OnEvent",function()
		if not NefpadDB then NefpadDB = { savedfiles = {} } end
		Nefpad.db = NefpadDB
	end)
end



--[[ GUI ]]--
function Nefpad:GUI()
	if self.frame then  -- it's already been built.
		if self.frame:IsVisible() then
			self.frame:Hide()
		else
			self.frame:Show()
		end
		return
	end
	
	-- Most of the following has been stolen and adapted from WowLua, by Cladhaire (yeah I'm that lazy).
	-- All art created by Beladona of WowInterface, taken from the addon WowLua.
	
	self.frame = CreateFrame("Frame","NefPadNoteFrame",UIParent)
	
	local frame = self.frame
	frame:SetHeight(self.db.height or 240)
	frame:SetWidth(self.db.width or 420)
	frame:SetPoint("CENTER")
	frame:EnableMouse(true)
	frame:SetFrameStrata("HIGH")
	frame:SetBackdrop({
		bgFile = self.path.."images\\backdrop.tga",
		edgeFile = self.path.."images\\borders.tga",
		tile = true, tileSize = 32, edgeSize = 16, 
		insets = { left = 16, right = 16, top = 16, bottom = 16 }
	})
	frame:SetScript("OnMouseUp",function(this)
		this.input:SetFocus()
	end)
	frame:SetMovable(true)
	
	frame.title = CreateFrame("Button",nil,frame)
	frame.title:SetHeight(32)
	frame.title:SetPoint("TOPLEFT",frame,"TOPLEFT",2,-2)
	frame.title:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-2,-2)
	frame.title:RegisterForDrag("LeftButton")
	frame.title:SetScript("OnDragStart",function() frame:StartMoving() end)
	frame.title:SetScript("OnDragStop",function() frame:StopMovingOrSizing() end)
	
	frame.headerL = frame.title:CreateTexture(nil,"ARTWORK")
	frame.headerL:SetTexture(self.path.."images\\headCorner.tga")
	frame.headerL:SetWidth(32)
	frame.headerL:SetHeight(32)
	frame.headerL:SetPoint("TOPLEFT",0,0)
	
	frame.headerR = frame.title:CreateTexture(nil,"ARTWORK")
	frame.headerR:SetTexture(self.path.."images\\headCorner.tga")
	frame.headerR:SetTexCoord(1,0,0,1)
	frame.headerR:SetWidth(32)
	frame.headerR:SetHeight(32)
	frame.headerR:SetPoint("TOPRIGHT",0,0)
	
	frame.header = frame.title:CreateTexture(nil,"ARTWORK")
	frame.header:SetTexture(self.path.."images\\header.tga")
	frame.header:SetPoint("TOPLEFT",frame.headerL,"TOPRIGHT")
	frame.header:SetPoint("BOTTOMRIGHT",frame.headerR,"BOTTOMLEFT")
	
	frame.titletext = frame.title:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
	frame.titletext:SetTextColor(1,1,1)
	frame.titletext:SetWidth(200)
	frame.titletext:SetHeight(16)
	frame.titletext:SetPoint("TOP",0,-2)
	
	frame.footerL = frame:CreateTexture(nil,"ARTWORK")
	frame.footerL:SetTexture(self.path.."images\\footCorner.tga")
	frame.footerL:SetWidth(32)
	frame.footerL:SetHeight(32)
	frame.footerL:SetPoint("BOTTOMLEFT",frame,"BOTTOMLEFT",2,2)
	
	frame.footerR = frame:CreateTexture(nil,"ARTWORK")
	frame.footerR:SetTexture(self.path.."images\\footCorner.tga")
	frame.footerR:SetTexCoord(1,0,0,1)
	frame.footerR:SetWidth(32)
	frame.footerR:SetHeight(32)
	frame.footerR:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-2,2)
	
	frame.footer = frame:CreateTexture(nil,"ARTWORK")
	frame.footer:SetTexture(self.path.."images\\footer.tga")
	frame.footer:SetPoint("TOPLEFT",frame.footerL,"TOPRIGHT")
	frame.footer:SetPoint("BOTTOMRIGHT",frame.footerR,"BOTTOMLEFT")
	
	frame.input = CreateFrame("EditBox","NefPadEditBox",frame)  -- must have a name because Blizzard is retarded.
	frame.input:SetMultiLine(true)
	frame.input:SetAutoFocus(false)
	frame.input:SetFont(self.path.."font.ttf",self.db.fontsize or 14)
	frame.input:EnableMouseWheel(true)
	frame.input:SetScript("OnMouseWheel",function(this,delta)
		if IsShiftKeyDown() then
			if delta > 0 then  -- scroll up.
				local size = (self.db.fontsize or 14) + 1
				if size > 32 then size = 32 end
				self.db.fontsize = size
				this:SetFont(self.path.."font.ttf",size)
			elseif delta < 0 then  -- scroll down.
				local size = (self.db.fontsize or 14) - 1
				if size < 4 then size = 4 end
				self.db.fontsize = size
				this:SetFont(self.path.."font.ttf",size)
			end
		else
			if delta > 0 then  -- scroll up.
				frame.scrollbar:SetValue(frame.scrollbar:GetValue() - 15)
			elseif delta < 0 then
				frame.scrollbar:SetValue(frame.scrollbar:GetValue() + 15)
			end
		end
	end)
	frame.input:SetScript("OnEscapePressed",function(this) this:ClearFocus() end)
	frame.input:SetScript("OnTabPressed",function(this)
		if this:HasFocus() then this:Insert("    ") end
	end)
	
	frame.scroll = CreateFrame("ScrollFrame","NefPadScrollFrame",frame,"UIPanelScrollFrameTemplate")  -- must have a name because Blizzard is retarded.
	frame.scroll:SetPoint("TOPLEFT",frame,"TOPLEFT",5,-30)
	frame.scroll:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-28,32)
	frame.scroll:SetScrollChild(frame.input)
	local throt = 0
	frame.input:SetScript("OnUpdate",function(this,elap)
		throt = throt + elap
		if throt >= .1 then
			throt = 0
			this:SetWidth(frame.scroll:GetWidth())
		end
	end)
	frame.scrollbar = NefPadScrollFrameScrollBar
	frame.input.max = 0
	frame.input:SetScript("OnTextChanged",function(this)
		local _,max = frame.scrollbar:GetMinMaxValues()
		if max > 0 and max ~= this.max then
			frame.scrollbar:SetValue(max)
			this.max = max
		end
	end)
	
	frame.resize = CreateFrame("Button",nil,frame)
	frame.resize:SetNormalTexture(self.path.."images\\resize.tga")
	frame.resize:SetHighlightTexture(self.path.."images\\resize.tga")
	frame.resize:SetWidth(20)
	frame.resize:SetHeight(20)
	frame.resize:SetPoint("BOTTOMRIGHT",0,0)
	frame.resize:SetScript("OnMouseDown",function() frame:StartSizing() end)
	frame.resize:SetScript("OnMouseUp",function()
		frame:StopMovingOrSizing()
		self.db.height = frame:GetHeight()  -- save the size.
		self.db.width = frame:GetWidth()
	end)
	frame.resize:SetScript("OnHide",function()
		frame:StopMovingOrSizing()
		self.db.height = frame:GetHeight()  -- save the size.
		self.db.width = frame:GetWidth()
	end)
	frame:SetResizable(true)
	frame:SetMinResize(300,120)
	
	frame.close = CreateFrame("Button",nil,frame.title,"UIPanelCloseButton")
	frame.close:SetHeight(25)
	frame.close:SetWidth(25)
	frame.close:SetPoint("TOPRIGHT",-5,3)
	frame.close:SetScript("OnClick",function() frame:Hide() end)
	local a,b,c = frame.close:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true); c:SetDesaturated(true)
	
	frame.new = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	frame.new:SetText(L.BUTTON_NEW)
	frame.new:SetHeight(18)
	frame.new:SetWidth(frame.new:GetTextWidth() + 20)
	frame.new:SetTextColor(1,1,1)
	frame.new:SetPoint("BOTTOMLEFT",10,1)
	frame.new:SetScript("OnClick",function() Nefpad:New() end)
	local _,a,b,c,d = frame.new:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true)
	c:SetDesaturated(true); d:SetDesaturated(true)
	
	frame.open = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	frame.open:SetText(L.BUTTON_OPEN)
	frame.open:SetHeight(18)
	frame.open:SetWidth(frame.open:GetTextWidth() + 20)
	frame.open:SetTextColor(1,1,1)
	frame.open:SetPoint("LEFT",frame.new,"RIGHT",4,0)
	local _,a,b,c,d = frame.open:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true)
	c:SetDesaturated(true); d:SetDesaturated(true)
	
	frame.dropdown = CreateFrame("Frame","NefpadDropdown",frame,"UIDropDownMenuTemplate")
	local function open() self:Open(this.value) end
	local info = {}
	UIDropDownMenu_Initialize(frame.dropdown,function()
		for key in pairs(self.db.savedfiles) do
			info.text = key
			info.value = key
			info.func = open
			UIDropDownMenu_AddButton(info)
		end
	end,"MENU")
	
	frame.open:SetScript("OnClick",function(this)
		ToggleDropDownMenu(1,nil,frame.dropdown,this,0,0)
	end)
	
	frame.save = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	frame.save:SetText(L.BUTTON_SAVE)
	frame.save:SetHeight(18)
	frame.save:SetWidth(frame.save:GetTextWidth() + 20)
	frame.save:SetTextColor(1,1,1)
	frame.save:SetPoint("LEFT",frame.open,"RIGHT",4,0)
	frame.save:SetScript("OnClick",function() Nefpad:Save() end)
	local _,a,b,c,d = frame.save:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true)
	c:SetDesaturated(true); d:SetDesaturated(true)
	
	frame.saveAs = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	frame.saveAs:SetText(L.BUTTON_SAVEAS)
	frame.saveAs:SetHeight(18)
	frame.saveAs:SetWidth(frame.saveAs:GetTextWidth() + 20)
	frame.saveAs:SetTextColor(1,1,1)
	frame.saveAs:SetPoint("LEFT",frame.save,"RIGHT",4,0)
	frame.saveAs:SetScript("OnClick",function() Nefpad:SaveAs() end)
	local _,a,b,c,d = frame.saveAs:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true)
	c:SetDesaturated(true); d:SetDesaturated(true)
	
	frame.delete = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	frame.delete:SetText(L.BUTTON_DELETE)
	frame.delete:SetHeight(18)
	frame.delete:SetWidth(frame.delete:GetTextWidth() + 20)
	frame.delete:SetTextColor(1,1,1)
	frame.delete:SetPoint("LEFT",frame.saveAs,"RIGHT",4,0)
	frame.delete:SetScript("OnClick",function() Nefpad:Delete() end)
	local _,a,b,c,d = frame.delete:GetRegions()
	a:SetDesaturated(true); b:SetDesaturated(true)
	c:SetDesaturated(true); d:SetDesaturated(true)
	
	-- insert it to be closed with escape.
	tinsert(UISpecialFrames,"NefPadNoteFrame")
	
	-- update the title bar.
	self:UpdateTitle()
	
	-- show it.
	frame:Show()
end

function Nefpad:UpdateTitle()
	self.frame.titletext:SetText(self.name.." v"..self.version.." - "..(self.note or "untitled"))
end



--[[ Popup dialogs ]]--
-- overwrite current file with blank?
StaticPopupDialogs.NEFPAD_OVERWRITE_NEW = {
	text = L.POPUP_OVERWRITE_UNSAVED,
	button1 = L.BUTTON_OKAY,
	button2 = L.BUTTON_CANCEL,
	OnAccept = function()
		Nefpad.note = nil
		Nefpad.frame.input:SetText("")
		Nefpad:UpdateTitle()
	end,
	whileDead = 1,
	hideOnEscape = 1,
	timeout = 0,
}

-- overwrite current file with open?
StaticPopupDialogs.NEFPAD_OVERWRITE_OPEN = {
	text = L.POPUP_OVERWRITE_UNSAVED,
	button1 = L.BUTTON_OKAY,
	button2 = L.BUTTON_CANCEL,
	OnAccept = function()
		Nefpad.note = Nefpad.open
		Nefpad.frame.input:SetText(Nefpad.db.savedfiles[Nefpad.open])
		Nefpad:UpdateTitle()
		Nefpad.open = nil
	end,
	whileDead = 1,
	hideOnEscape = 1,
	timeout = 0,
}

-- save a file.
StaticPopupDialogs.NEFPAD_SAVEAS = {
	text = L.POPUP_SAVEAS,
	button1 = L.BUTTON_OKAY,
	button2 = L.BUTTON_CANCEL,
	OnAccept = function()
		local name = _G[this:GetParent():GetName().."EditBox"]:GetText()
		if Nefpad.db.savedfiles[name] then
			Nefpad.saveas = name
			StaticPopup_Show("NEFPAD_SAVEAS_EXISTS")
		else
			Nefpad.note = name
			Nefpad.db.savedfiles[name] = Nefpad.frame.input:GetText()
			Nefpad:UpdateTitle()
		end
	end,
	OnHide = function()
		_G[this:GetName().."EditBox"]:SetText("")  -- it seems it doesn't auto-clear itself, for some reason.
	end,
	whileDead = 1,
	hideOnEscape = 1,
	hasEditBox = 1,
	timeout = 0,
}

-- overwrite the previous file of the same name? (saveas).
StaticPopupDialogs.NEFPAD_SAVEAS_EXISTS = {
	text = L.POPUP_OVERWRITE_PREVSAVE,
	button1 = L.BUTTON_OKAY,
	button2 = L.BUTTON_CANCEL,
	OnAccept = function()
		Nefpad.note = Nefpad.saveas
		Nefpad.db.savedfiles[Nefpad.saveas] = Nefpad.frame.input:GetText()
		Nefpad:UpdateTitle()
		Nefpad.saveas = nil
	end,
	whileDead = 1,
	hideOnEscape = 1,
	timeout = 0,
}

-- are you sure you want to delete?
StaticPopupDialogs.NEFPAD_DELETE_CONFIRM = {
	text = L.POPUP_DELETE,
	button1 = L.BUTTON_OKAY,
	button2 = L.BUTTON_CANCEL,
	OnAccept = function()
		Nefpad.db.savedfiles[Nefpad.note] = nil
		Nefpad.note = nil
		Nefpad.frame.input:SetText("")
		Nefpad:UpdateTitle()
	end,
	whileDead = 1,
	hideOnEscape = 1,
	timeout = 0,
	showAlert = 1,
}



--[[ "File" handling ]]--
function Nefpad:Save()
	if self.note then
		self.db.savedfiles[self.note] = self.frame.input:GetText()  -- if we're handling previously saved note, simply save it to the DB as-is.
	else
		self:SaveAs()  -- if the note has not been previously saved and save is clicked, run :SaveAs().
	end
end

function Nefpad:SaveAs()
	StaticPopup_Show("NEFPAD_SAVEAS")
end

function Nefpad:New()
	local text = self.frame.input:GetText()
	if (self.note and self.db.savedfiles[self.note] == text) or text == "" then
		self.frame.input:SetText("")
		self.note = nil
		self:UpdateTitle()
	else
		StaticPopup_Show("NEFPAD_OVERWRITE_NEW")
	end
end

function Nefpad:Open(file)
	local fileText = self.db.savedfiles[file]
	if not fileText then return end
	
	local text = self.frame.input:GetText()
	if (self.note and self.db.savedfiles[self.note] == text) or text == "" then
		self.frame.input:SetText(fileText)
		self.note = file
		self:UpdateTitle()
	else
		self.open = file
		StaticPopup_Show("NEFPAD_OVERWRITE_OPEN")
	end
end

function Nefpad:Delete()
	if self.note then
		StaticPopup_Show("NEFPAD_DELETE_CONFIRM",self.note)
	end
end



--[[ Hooks ]]--
hooksecurefunc("HandleModifiedItemClick",function(link)
	if Nefpad.frame and Nefpad.frame.input:HasFocus() and IsModifiedClick("CHATLINK") then
		Nefpad.frame.input:Insert(link)
	end
end)



--[[ Slash cmd ]]--
SLASH_NEFPAD1 = "/nefpad"
SLASH_NEFPAD2 = "/np"
function SlashCmdList.NEFPAD()
	if Nefpad and Nefpad.GUI then
		Nefpad:GUI()
	end
end
